/*=========================================================
ComVar.h
Part of MAM-BP

Type 0 "LED Fixe" avec les règles annoncées
Type 1 "LED Clignotante". 
Type 2 "LED Fading". 
Le CV CV_FADING_LEDS = 5 fixe la vitesse de FADING commune (1/2 période en 1/10 s). Avec la même fréquence ON/OFF pour toutes.
Le CV CV_BLINK_LEDS = 10 fixe la vitesse de clignotement commune (1/2 période en 1/10 s). Avec la même fréquence ON/OFF pour toutes.
Dans les 2 cas Temporisation permet d'éteindre la LED en fin de temporisation. Pas d'extinction si temporisation à 0.

Type 5 "solenoide". 
Les CV 3, 4, 5, 6 controlent la durée d'activation des solenoides de 0.1s à 25.5s. 0 = activation permanente

Si SlowMotion Servo est utilisé, à cause du manque de mémoire :
    Le mode est analogique si Min<>Max et si la Pin Arduino le permet : deux pins impossibles, J1 et J7. 
Sinon
    Le mode est analogique si Min<>Max pour toutes les Pins J1 à J8.
    Le fading est alors possible à la montée comme à la descente

Servomoteur :
  Le passage de 0 à 1 met le servo en position Max 
  Le passage de 1 à 0 met le servomoteur en position Min
LED Fixe ou clignotante :
  Le passage de 0 à 1 allume la LED ou la met au Max quel que soit la tempo
  Le passage de 1 à 0 éteint la LED ou la met au Min si tempo=0
  Le passage de 1 à 0 éteint la LED ou la met au Min en fin de décompte si tempo>0 donc immédiatement si tempo=0, 
Liste de MP3 :
  Le passage de 0 à 1 de cette adresse déclencherait la fonction    
  Le passage de 1 à 0 sera ignoré

Les commandes USB, BP et DCC sont systématiquement traduites en id, enregistrée dans BP[id]

Si la directive UseRedirections est definie:
BP   DCC(Base+)    Fct                          CV                        Appui court         Appui Long        Appui très long
BP1                B1                                 Fonction N°13       Detection DCC       Enreg DCC         ON / OFF Boutons Poussoir
                                                                          /Retour normal
BP2                B2                                 Fonction N°14       ON = 1              OFF = 0  
BP3                B3                                 Fonction N°15       ON = 1              OFF = 0  
BP4                B4-Canton                          Fonction N°16       ON = 1              OFF = 0  
        0     1    J1 servo/sol ou LED (PWM)    05x   Fonction N°1        servo au min = 0    servo au MAX = 1
        1     2    J2 servo/sol ou LED          06x   Fonction N°2        servo au min        servo au MAX
        2     3    J3 servo/sol ou LED (PWM)    07x   Fonction N°3        servo au min        servo au MAX
        3     4    J4 servo/sol ou LED (PWM)    08x   Fonction N°4        servo au min        servo au MAX
        4     5    J5 servo/sol ou LED (PWM)    09x   Fonction N°5        servo au min        servo au MAX
        5     6    J6 servo/sol ou LED (PWM)    10x   Fonction N°6        servo au min        servo au MAX
        6     7    J7 servo/sol ou LED (PWM)    11x   Fonction N°7        servo au min        servo au MAX
        7     8    J8 servo/sol ou LED          12x   Fonction N°8        servo au min        servo au MAX
AK1     8     9                                 15x   Liste MP3 N°1       ON = 1              OFF = 0
AK2     9     10                                16x   Liste MP3 N°2       ON = 1              OFF = 0
AK3     10    11                                17x   Liste MP3 N°3       ON = 1              OFF = 0
AK4     11    12                                18x   Liste MP3 N°4       ON = 1              OFF = 0
AK5     12    13                                19x   Liste MP3 N°5       ON = 1              OFF = 0
AK6     13    14                                20x   Liste MP3 N°6       ON = 1              OFF = 0
AK7     14    15                                21x   Liste MP3 N°7       ON = 1              OFF = 0
AK8     15    16                                22x   Liste MP3 N°8       ON = 1              OFF = 0

la directive UseBasicRedir ne prends en compte que l'adresse de base des BP et l'adresse de base des BP. Les autres suivent.

Pour chaque bouton/Adresse DCC
L'adresse DCC vaut un appui court (passage de 0 à 1) ou long (passage de 1 à 0).

Les commandes USB sont réservées au test et à la mise en œuvre. En fonctionnement normal, on preferera boutons ou DCC.

===========================================================*/

#include "Arduino.h"

// ========================================================
// Variables et constantes
// ========================================================

// Pin Inputs (BP / Occ)
#define BP1Pin          14              // A0 BP1
#define BP2Pin          15              // A1 BP2
#define BP3Pin          16              // A2 BP3
#define AdKeyPin        20              // A6 ADKey
#define OccPin          21              // A7 Cantons

// definition des types de focntions
#define TypeFixLed      0               // LED Fixe
#define TypeBlinkLed    1               // LED Cligottante
#define TypeFading      2               // LED Fading
#define TypeMP3List     3               // Liste MP3
#define TypeServo       4               // Servomoteur
#define Typesolenoid    5               // Solenoïde

// limite de temps pour l'appui des BP
#define CptTcoLimit     15              // Appui court/long BP = n * 50ms
#define CptOnOffLimit   50              // Appui long/tres long BP

// Parametrage DCC
#define MaxCV           250             // Nombre de CV affichable, ne pas dépasser 255 !

#define CV_Base_Reset    30             // Premier CV a reseter

#if defined(MAM_BP)
  #if defined(SlowMotionSrv)            // >>>>> Utilisation de la bibliotheque SlowMotionServo
    #define MaxDccAddress 16            // maximum 16 !
  #else
    #define MaxDccAddress 20            // maximum 20 !
  #endif
#elif defined(MAM_PN)
  #define MaxDccAddress    8            // 8 adresses Dcc possibles : Offset / adresse de base (0..7)
#endif

#define AccOffset       0               // Offset des adresses DCC Lenz et Roco : 4
#define OffsetFct       1               // permet d'afficher les fonctions de 0 à 7 ou de 1 à 8

// Gestion de la communication PC via USB
String inputString = "";                // Chaine permettant de recevoir les informations du PC via USB
boolean stringComplete = false;         // Drapeau indiquant que la chaine recue est complete (finit par CR)

// Generalité 
boolean BP_actif=true;                  // BP actifs

// Gestion du son
uint16_t Volume=5;                      // Sound volume
uint8_t  Attenuation = 5;               // Sound volume

// Play List MP3
static uint8_t PL[10];                  // Elements de Playlist
uint8_t PLi=10;                         // Pointeur de PlayList

// N° commande collectée Dcc, Inputs, USB
int     id=-1;

// Etat des commandes recues DCC USB ou autres
// Etat Interrupteurs, BP et Occupations
#if defined(MAM_PN)
  boolean         BP[] = { true,true,true,true,false};
#else
  #if defined(SlowMotionSrv)
// Utilisation de la bibliotheque SlowMotionServo, 16/20 adresses DCC
  static boolean  BP[] = { true,true,true,true
                          ,true,true,true,true
                          ,true,true,true,true
                          ,true,true,true,true };
  #else
  static boolean  BP[] = { true,true,true,true
                          ,true,true,true,true
                          ,true,true,true,true
                          ,true,true,true,true
                          ,true,true,true,true };
  #endif
#endif

// Temporisations
uint32_t   currentMillis = 0;
uint8_t    CptTco  = 0;                 // Temporisation appui court/long/tres long BP
#define mMillisLimit       50
uint32_t   mMillis = 0;                 // Init Temporisation MP3

uint16_t   bMillisLimit = 1000;
uint32_t   bMillis = 0;                 // Init Clignotement LEDs
boolean    BlinkLed = false;
#define pMillisLimit           1000
uint32_t   pMillis = 0;                 // Init Temporisation LEDs
static uint8_t  Tp[] = { 0,0,0,0
                        ,0,0,0,0 };
uint16_t   Cpt=0;                       // Timeout MP3 = 10 sec

#define pCpMillisLimit         1000
uint32_t   pCpMillis = 0;               // Init Temporisation casse pieds
uint8_t    DelaiCassePieds = 10;        // Temporisation anti casse-pieds
uint8_t    CptCassePieds=0;             // Pour anti casse-pieds

// Servomoteurs
#if defined(SlowMotionSrv)              // >>>>> Utilisation de la bibliotheque SlowMotionServo
  #include <SlowMotionServo.h>
#endif

// ========================================================
// Structures BP
// ========================================================

struct iBP
{
  uint8_t       Type;                   // 0=LED, 1=LED clignotante, 2=LED fading, 3=MP3, 4=servo, 5=solenoid // CV 0y0 avec y = 10*(5+i)
  uint8_t       Pin;                    // Pin                                             CV 0y1
  uint8_t       Min;                    // Min                                             CV 0y2
  uint8_t       Max;                    // Max                                             CV 0y3
  uint8_t       Speed;                  // Vitesse                                         CV 0y4
  uint8_t       Angle;                  // Angle actuel
  uint8_t       Objectif;               // Angle objectif
  bool          Blink;                  // Clignotement

#if defined(SlowMotionSrv)              // >>>>> Utilisation de la bibliotheque SlowMotionServo
  #if defined(Servo_Linear)
    SMSLinear myServo;
  #endif
  #if defined(Servo_Smooth)
    SMSSmooth myServo;
  #endif
#endif
};
static          iBP BPs[8];             // Definition des fonctions

// Boutons multiples
const int       SeuilADKEY[] = { 980,890,820  // 13 boutons
                                ,750,700,650
                                ,610,575,545
                                ,510,480,425,0 };
const uint8_t   JPins[] = { 3,4,5,6,9,10,11,12 };
                                
