/*==========================================================
Dcc.h                           N° CV
============================================================*/

#define CV_ACCESSORY_DECODER_ADDRESS_LSB 1
#define CV_ACCESSORY_DECODER_ADDRESS_MSB 9
#define CV_ATTENUATION            14
#define CV_VOLUME                 15
#define CV_CASSE_PIEDS            19
#define CV_FREQ_CLIGN             20

// ========================================================
// Structures DCC
// ========================================================
struct CVPair
{
  uint8_t  CV;
  uint8_t  Value;
};

// --------------------------------------------------------
#if defined(MAM_BP) || defined(MAM_PN)
  #define CV_FADING_LEDS          10   // vitesse de fading commune en 1/10s
  #define CV_BLINK_LEDS           20   // vitesse de clignotement commune en 1/10s
  #define CV_Base_REDIR_AK        30   // 30 à 45 (16 AdKeys)
  #define CV_Base_REDIR_BP        46   // 46 à 49 (4  Interrupteurs BP)

  #define CV_Base_8Fct            50   // 50  à 129 (8 fonctions)
  #define CV_Base_MP3            150   // 150 à 229 (8 listes de MP3)
// --------------------------------------------------------
#elif defined(MAM_PN)
// N° Evénements donnés par les collecteurs Dcc btn ...
  #define PNoccupe              1001
  #define PNlibre               1002

  #define CV_NBRE_BARRIERE        13   // 2 ou 4
  #define CV_FERMETURE            16   // en s
  #define CV_SILENCE              17   // en s
  #define CV_PN_STATUS            18   // fermé = 1, ouvert = 0
  #define CV_BLINK_LEDS           20   // vitesse de clignotement commune en 1/10s
  #define CV_Base_8Fct            50   // 50  à 129 (8 fonctions)
  #define CV_Base_MP3            130   // 130 à 249 (12 listes de MP3)
#endif

// ========================================================
// Structures DCC adresse      N° CV
// ========================================================

/*
#define CV_HMIN                 22
#define CV_HMAX                 23
#define CV_LIGHTLIMIT           24
#define CV_LATITUDE             25
#define CV_DAY                  26
#define CV_CLOCK_SPEED          27
#define CV_FADING_SPEED         28
#define CV_MP3CLOCHE            30
*/

#define CV_VITESSE_SERVOS       11
#define CV_SENS_SERVOS          12

#define CV_VITESSE_SERVO_1      54  // Servo
#define CV_ANGLE_MINI_SERVO_1   52
#define CV_ANGLE_MAXI_SERVO_1   53

#define CV_VITESSE_SERVO_2      64
#define CV_ANGLE_MINI_SERVO_2   62
#define CV_ANGLE_MAXI_SERVO_2   63

#define CV_VITESSE_SERVO_3      74
#define CV_ANGLE_MINI_SERVO_3   72
#define CV_ANGLE_MAXI_SERVO_3   73

#define CV_VITESSE_SERVO_4      84
#define CV_ANGLE_MINI_SERVO_4   82
#define CV_ANGLE_MAXI_SERVO_4   83

#define CV_VITESSE_SERVO_5      94
#define CV_ANGLE_MINI_SERVO_5   92
#define CV_ANGLE_MAXI_SERVO_5   93

#define CV_VITESSE_SERVO_6      104
#define CV_ANGLE_MINI_SERVO_6   102
#define CV_ANGLE_MAXI_SERVO_6   103

#define CV_VITESSE_SERVO_7      114
#define CV_ANGLE_MINI_SERVO_7   112
#define CV_ANGLE_MAXI_SERVO_7   113

#define CV_VITESSE_SERVO_8      124
#define CV_ANGLE_MINI_SERVO_8   122
#define CV_ANGLE_MAXI_SERVO_8   123

#define CV_BP_ACTION_1_0        150  // MP3
#define CV_BP_ACTION_1_1        160
#define CV_BP_ACTION_2_0        170
#define CV_BP_ACTION_2_1        180
#define CV_BP_ACTION_3_0        190
#define CV_BP_ACTION_3_1        200

#define CV_ADKEY_MP3_1          130
#define CV_ADKEY_MP3_2          140
#define CV_ADKEY_MP3_3          150
#define CV_ADKEY_MP3_4          160
#define CV_ADKEY_MP3_5          170
#define CV_ADKEY_MP3_6          180
#define CV_ADKEY_MP3_7          190
#define CV_ADKEY_MP3_8          200
#define CV_ADKEY_MP3_9          210
#define CV_ADKEY_MP3_10         220
#define CV_ADKEY_MP3_11         230
#define CV_ADKEY_MP3_12         240

// ========================================================
// Structures DCC data
// ========================================================

#if defined(MAM_PN)

CVPair FactoryDefaultCVs [] =
{
  // Standard
  {CV_ACCESSORY_DECODER_ADDRESS_LSB, 1},        // Adresse DCC (low byte)
  {CV_ACCESSORY_DECODER_ADDRESS_MSB, 0},        // Adresse DCC (high byte)
  {2,255},      // Validation des 8 sorties
//  {7,41},       // Révision (read only)                             // written by NMRAdcc init
//  {8,13},       // Constructeur (read only)                         // written by NMRAdcc init
//  {29,128},     // Configuration (128 = décodeur d'accessoire)      // written by NMRAdcc init
                                                                      // also write CV48 & CV49
  // Non standard
  {CV_VITESSE_SERVOS,7},        // Vitesse commune des servomoteurs
  {CV_SENS_SERVOS,0},           // Sens des servomoteurs (bits)
  {CV_NBRE_BARRIERE,4},         // 2 ou 4
  {CV_ATTENUATION,5},           // Volume Sonnerie attenuée
  {CV_VOLUME,10},               // Volume Son MP3
  {CV_FERMETURE,7},             // Delai Avertissement Fermeture en s
  {CV_SILENCE,12},              // Delai Avertissement Silence en s
  {CV_PN_STATUS,0},             // PN Occupé
  {CV_CASSE_PIEDS,10},          // Delai anti casse-pieds en s
  {CV_BLINK_LEDS,5},            // Delai Clignottement avertissement en 1/10s

  {CV_Base_8Fct+40,1},          //  Fct4 LED clignotante
  {CV_Base_8Fct+41,5},          //  Fct4 Pin 9 = J5

  {CV_Base_8Fct+50,1},          //  Fct5 LED clignotante
  {CV_Base_8Fct+51,6},          //  Fct5 Pin 10 = J6

  {CV_Base_8Fct+60,1},          //  Fct6 LED clignotante
  {CV_Base_8Fct+61,7},          //  Fct6 Pin 11 = J7

  {CV_Base_8Fct+70,1},          //  Fct7 LED clignotante
  {CV_Base_8Fct+71,8},          //  Fct7 Pin 12 = J8

  {CV_Base_8Fct,4},             //  Fct1 Servo
  {CV_Base_8Fct+1,1},           //  Fct1 Pin 3 = J1
//  {CV_Base_8Fct+2,80},          //  Fct1 Min
//  {CV_Base_8Fct+3,120},         //  Fct1 Max
//  {CV_Base_8Fct+4,7},           //  Fct1 Vitesse Servo

  {CV_Base_8Fct+10,4},          //  Fct2 Servo
  {CV_Base_8Fct+11,2},          //  Fct2 Pin 4 = J2
//  {CV_Base_8Fct+12,80},         //  Fct2 Min
//  {CV_Base_8Fct+13,120},        //  Fct2 Max
//  {CV_Base_8Fct+14,7},          //  Fct2 Vitesse Servo

  {CV_Base_8Fct+20,4},          //  Fct3 Servo
  {CV_Base_8Fct+21,3},          //  Fct3 Pin 5 = J3
//  {CV_Base_8Fct+22,80},         //  Fct3 Min
//  {CV_Base_8Fct+23,120},        //  Fct3 Max
//  {CV_Base_8Fct+24,7},          //  Fct3 Vitesse Servo

  {CV_Base_8Fct+30,4},          //  Fct4 Servo
  {CV_Base_8Fct+31,4},          //  Fct4 Pin 6 = J4
//  {CV_Base_8Fct+32,80},         //  Fct4 Min
//  {CV_Base_8Fct+33,120},        //  Fct4 Max
//  {CV_Base_8Fct+34,7},          //  Fct4 Vitesse Servo

  {CV_ANGLE_MINI_SERVO_1,80},   //  Servo 1: Angle maximum
  {CV_ANGLE_MAXI_SERVO_1,120},  //  Servo 1: Angle minimum
  {CV_VITESSE_SERVO_1,7},       //  Servo 1: Vitesse du servo

  {CV_ANGLE_MINI_SERVO_2,80},   //  Servo 2: Angle maximum
  {CV_ANGLE_MAXI_SERVO_2,120},  //  Servo 2: Angle minimum
  {CV_VITESSE_SERVO_2,7},       //  Servo 2: Vitesse du servo

  {CV_ANGLE_MINI_SERVO_3,80},   //  Servo 3: Angle maximum
  {CV_ANGLE_MAXI_SERVO_3,120},  //  Servo 3: Angle minimum
  {CV_VITESSE_SERVO_3,7},       //  Servo 3: Vitesse du servo

  {CV_ANGLE_MINI_SERVO_4,80},   //  Servo 4: Angle maximum
  {CV_ANGLE_MAXI_SERVO_4,120},  //  Servo 4: Angle minimum
  {CV_VITESSE_SERVO_4,7},       //  Servo 4: Vitesse du servo

//  {CV_BP_ACTION_1_0,150},       //  Son BP 1 rising
//  {CV_BP_ACTION_1_1,170},       //  Son BP 1 falling
//  {CV_BP_ACTION_2_0,151},       //  Son BP 2 rising
//  {CV_BP_ACTION_2_1,171},       //  Son BP 2 falling
//  {CV_BP_ACTION_3_0,152},       //  Son BP 3 rising
//  {CV_BP_ACTION_3_1,173},       //  Son BP 3 falling

  {CV_ADKEY_MP3_1,7},           //  Son AdKey 1
  {CV_ADKEY_MP3_2,201},         //  Son AdKey 2
  {CV_ADKEY_MP3_3,202},         //  Son AdKey 3
  {CV_ADKEY_MP3_4,204},         //  Son AdKey 4
  {CV_ADKEY_MP3_5,205},         //  Son AdKey 5
  {CV_ADKEY_MP3_6,150},         //  Son AdKey 6
  {CV_ADKEY_MP3_7,151},         //  Son AdKey 7
  {CV_ADKEY_MP3_8,171},         //  Son AdKey 8
  {CV_ADKEY_MP3_9,172},         //  Son AdKey 9
  {CV_ADKEY_MP3_10,173},        //  Son AdKey 10
  {CV_ADKEY_MP3_11,1},          //  Son AdKey 11
  {CV_ADKEY_MP3_12,7},          //  Son AdKey 12

};
#endif // MAM_PN
// --------------------------------------------------------
#if defined(MAM_BP)

CVPair FactoryDefaultCVs [] =
{
  // Standard
  {CV_ACCESSORY_DECODER_ADDRESS_LSB, 1},        // Adresse DCC (low byte)
  {CV_ACCESSORY_DECODER_ADDRESS_MSB, 0},        // Adresse DCC (high byte)
  {2,255},      // Validation des 8 sorties
  {3,5},        // Durée d'activation des Solénoïdes J1 et J2
  {4,5},        // Durée d'activation des Solénoïdes J3 et J4
  {5,5},        // Durée d'activation des Solénoïdes J5 et J6
  {6,5},        // Durée d'activation des Solénoïdes J7 et J8
//  {7,43},       // Révision (read only)                             // written by NMRAdcc init
//  {8,13},       // Constructeur (read only)                         // written by NMRAdcc init
//  {29,128},     // Configuration (128 = décodeur d'accessoire)      // written by NMRAdcc init
  // Non standard
  {CV_FADING_LEDS, 6},          // 1/2 periode de fading LED 1/10 secondes
  {CV_VOLUME,5},                // Volume Son MP3
  {CV_CASSE_PIEDS,10},          // Temporisation anti casse-pieds en secondes
  {CV_BLINK_LEDS, 6},           // 1/2 periode Clignotement LED 1/10 secondes

#if defined(UseRedirections)
  {CV_Base_REDIR_AK, 1},        // Fonction associee a AdKey1
  {CV_Base_REDIR_AK+1, 2},      // Fonction associee a AdKey2
  {CV_Base_REDIR_AK+2, 3},      // Fonction associee a AdKey3
  {CV_Base_REDIR_AK+3, 4},      // Fonction associee a AdKey4
  {CV_Base_REDIR_AK+4, 5},      // Fonction associee a AdKey5
  {CV_Base_REDIR_AK+5, 6},      // Fonction associee a AdKey6
  {CV_Base_REDIR_AK+6, 7},      // Fonction associee a AdKey7
  {CV_Base_REDIR_AK+7, 8},      // Fonction associee a AdKey8
  {CV_Base_REDIR_AK+8, 9},      // Fonction associee a AdKey9
  {CV_Base_REDIR_AK+9, 10},     // Fonction associee a AdKey10
  {CV_Base_REDIR_AK+10, 11},    // Fonction associee a AdKey11
  {CV_Base_REDIR_AK+11, 12},    // Fonction associee a AdKey12
  {CV_Base_REDIR_AK+12, 13},    // Fonction associee a AdKey13
  {CV_Base_REDIR_AK+13, 14},    // Fonction associee a AdKey14
  {CV_Base_REDIR_AK+14, 15},    // Fonction associee a AdKey15
  {CV_Base_REDIR_AK+15, 16},    // Fonction associee a AdKey16

  {CV_Base_REDIR_BP, 17},       // Fonction associee a BP1
  {CV_Base_REDIR_BP+1, 18},     // Fonction associee a BP2
  {CV_Base_REDIR_BP+2, 19},     // Fonction associee a BP3
  {CV_Base_REDIR_BP+3, 20},     // Fonction associee a BP4 (Occupation de canton)
#elif defined(UseBasicRedir)
  {CV_Base_REDIR_AK, 1},        // Fonction associee aux AdKey
  {CV_Base_REDIR_BP, 13},       // Fonction associee aux BP
#endif

  {CV_Base_8Fct,0},             //  Fct0 LED fixe
  {CV_Base_8Fct+1,1},           //  Fct0 Pin 3 = J1
  {CV_Base_8Fct+2,255},         //  Fct0 Min
  {CV_Base_8Fct+3,255},         //  Fct0 Max
  {CV_Base_8Fct+4,0},           //  Fct0 LED tempo 0

  {CV_Base_8Fct+10,1},          //  Fct1 LED clignotante
  {CV_Base_8Fct+11,2},          //  Fct1 Pin 4 = J2
  {CV_Base_8Fct+12,255},        //  Fct1 Min
  {CV_Base_8Fct+13,255},        //  Fct1 Max
  {CV_Base_8Fct+14,9},          //  Fct1 LED tempo 9s

  {CV_Base_8Fct+20,1},          //  Fct2 LED clignotante
  {CV_Base_8Fct+21,3},          //  Fct2 Pin 5 = J3
  {CV_Base_8Fct+22,90},         //  Fct2 Min
  {CV_Base_8Fct+23,220},        //  Fct2 Max
  {CV_Base_8Fct+24,20},         //  Fct2 LED tempo 20s

  {CV_Base_8Fct+30,2},          //  Fct3 LED Fading
  {CV_Base_8Fct+31,4},          //  Fct3 Pin 6 = J4
  {CV_Base_8Fct+32,20},         //  Fct3 Min
  {CV_Base_8Fct+33,200},        //  Fct3 Max
  {CV_Base_8Fct+34,0},          //  Fct3 LED tempo 0
  {CV_Base_8Fct+35,6},          //  Fct3 LED vitesse 6

  {CV_Base_8Fct+40,5},          //  Fct4 Solénoïde
  {CV_Base_8Fct+41,5},          //  Fct4 Pin 9 = J5
  {CV_Base_8Fct+42,255},        //  Fct4 Min
  {CV_Base_8Fct+43,255},        //  Fct4 Max
  {CV_Base_8Fct+44,5},          //  Fct4 LED tempo 0.5

  {CV_Base_8Fct+50,5},          //  Fct5 Solénoïde
  {CV_Base_8Fct+51,6},          //  Fct5 Pin 10 = J6
  {CV_Base_8Fct+52,255},        //  Fct5 Min
  {CV_Base_8Fct+53,255},        //  Fct5 Max
  {CV_Base_8Fct+54,5},          //  Fct5 Tempo 0.5

  {CV_Base_8Fct+60,3},          //  Fct6 MP3
  {CV_Base_8Fct+61,171},        //  Fct6 Fichier MP3

  {CV_Base_8Fct+70,4},          //  Fct7 Servo
  {CV_Base_8Fct+71,8},          //  Fct7 Pin 12 = J8
  {CV_Base_8Fct+72,70},         //  Fct7 Min
  {CV_Base_8Fct+73,110},        //  Fct7 Max
  {CV_Base_8Fct+74,1},          //  Fct7 Vitesse Servo

#if !defined(SlowMotionSrv)     //  Le fading n'est possible que si SlowMotionSrv est désactivé
  {CV_Base_8Fct+25,10},         //  Fct2 FadeUp
  {CV_Base_8Fct+26,20},         //  Fct2 FadeDown

  {CV_Base_8Fct+35,1},          //  Fct3 FadeUp
  {CV_Base_8Fct+36,2},          //  Fct3 FadeDown

  {CV_Base_8Fct+45,1},          //  Fct4 FadeUp
  {CV_Base_8Fct+46,2},          //  Fct4 FadeDown

  {CV_Base_8Fct+55,1},          //  Fct5 FadeUp
  {CV_Base_8Fct+56,2},          //  Fct5 FadeDown
#endif

  {CV_Base_MP3+0,1},            //  Fct8 Liste de son 1, son 1
  {CV_Base_MP3+1,3},            //  Fct8 Liste de son 1, son 2
  {CV_Base_MP3+2,97},           //  Fct8 Liste de son 1, son 3
  {CV_Base_MP3+3,4},            //  Fct8 Liste de son 1, son 4
  {CV_Base_MP3+4,70},           //  Fct8 Liste de son 1, son 5
  {CV_Base_MP3+5,5},            //  Fct8 Liste de son 1, son 6
  {CV_Base_MP3+6,51},           //  Fct8 Liste de son 1, son 7
  {CV_Base_MP3+7,6},            //  Fct8 Liste de son 1, son 8
  {CV_Base_MP3+8,255},          //  Fct8 Liste de son 1, Fin

  {CV_Base_MP3+10,2},           //  Fct9 Liste de son 2, son 1
  {CV_Base_MP3+11,61},          //  Fct9 Liste de son 2, son 2
  {CV_Base_MP3+12,7},           //  Fct9 Liste de son 2, son 3
  {CV_Base_MP3+13,255},         //  Fct9 Liste de son 2, Fin

  {CV_Base_MP3+20,2},           //  Fct10 Liste de son 3, son 1
  {CV_Base_MP3+21,12},          //  Fct10 Liste de son 3, son 2
  {CV_Base_MP3+22,72},          //  Fct10 Liste de son 3, son 3
  {CV_Base_MP3+23,20},          //  Fct10 Liste de son 3, son 3
  {CV_Base_MP3+24,255},         //  Fct10 Liste de son 4, Fin
};

#endif // MAM_BP
