/*=========================================================
Dcc_Std.h
===========================================================*/
#include "Dcc.h"
#include "Lcd.h"
#include <NmraDcc.h>
extern void Track_Play(int Track, String Name);

// ================================================================================================================
// Routines NMRA DCC
// Now works with both short and long DCC Addesses for CV Control Default 24 (LSB CV 121 ; MSB CV 122) >>>> CV 248
// ================================================================================================================
NmraDcc  Dcc ;
DCC_MSG  Packet ;
uint8_t  FactoryDefaultCVIndex = 0;

#define  DccAckPin                         LED_BUILTIN
#define  CV_To_Store_SET_CV_Address        248 // 0xF8
#define  CV_Accessory_Address              (CV_ACCESSORY_DECODER_ADDRESS_LSB & 0xFC) & (CV_ACCESSORY_DECODER_ADDRESS_LSB&7)<<8

// ========================================================
// Variables DCC
// ========================================================
uint8_t Mode=0;                         // Normal (=0) Detection Adresse Dcc en cours(=1)
int DccAddress=-1;                      // Adresse decodeur accessoire Dcc
int tmpDccAddress=0;

// ========================================================
// setup DCC Decoder
// ========================================================
void setupDCCDecoder()
{
  Dcc.pin(0, 2, 1);                     // Setup which External Interrupt, the Pin it's associated with that we're using and enable the Pull-Up
  // Call the main DCC Init function to enable the DCC Receiver
  Dcc.init( MAN_ID_DIY, MAM_Version, FLAGS_OUTPUT_ADDRESS_MODE | FLAGS_DCC_ACCESSORY_DECODER, CV_To_Store_SET_CV_Address);
}

#if defined(USE_DCC)                    // Commandes DCC

// ========================================================
//   unused
// ========================================================
void notifyCVAck(void)
{
  Serial.println(F("notifyCVAck")) ;
  digitalWrite( DccAckPin, HIGH );
  delay(6);
  digitalWrite( DccAckPin, LOW );
}

// ========================================================
// Comment/Uncomment to unprint/print all DCC Packets
// ========================================================
#ifdef  NOTIFY_DCC_MSG
void notifyDccMsg( DCC_MSG * Msg)
{
  Serial.print(F("notifyDccMsg: ")) ;
  for(uint8_t i = 0; i < Msg->Size; i++)
  {
    Serial.print(Msg->Data[i], HEX);
    Serial.write(' ');
  }
  Serial.println();
}
#endif

// ========================================================
// Decode la trame DCC recue
// ========================================================
void DecodeDcc(uint16_t address,boolean enable) {
  int adDCC=address-DccAddress;
  if (adDCC>=0 && adDCC<MaxDccAddress) {
    Sprint_F(AccDcc);
    Serial.print(address);
    Serial.print(F(" (Fct"));
    Serial.print(adDCC+1);
    Serial.print(F(") = "));
    Serial.println(enable);
    BP[adDCC+1]=enable;              // shifted by 1 !
    id=adDCC;
    #if defined(Debug)
      Serial.print(adDCC);
      Serial.print(F(" DCC ..."));
      Serial.print(adDCC);
      Serial.print(F(" Id="));
      Serial.println(id);
      Serial.println();
    #endif
  }
  #if defined(Debug)
  else {
    Serial.print(DccAddress);
    Serial.print("-");
    Serial.print(DccAddress + MaxDccAddress);
    Sprint_F(OtherDcc);
    Sprint_F(ExpectedDcc);
    Serial.println(address);
    Serial.println();
  }
  #endif
}

// ========================================================
// Dcc Accessory Signal message processing (extended address)
// ========================================================

void notifyDccSigState( uint16_t Addr, uint8_t OutputIndex, uint8_t Data )
{
  uint16_t dccAdr = (Addr & 0x7FF) + 3;
  Serial.print(F("Accessory Signaling with extended address "));
  Serial.print(dccAdr,DEC);
  Serial.print(F(", data "));
  Serial.println(Data,DEC);
}

// ========================================================
// Dcc Accessory Turnout message processing
// ========================================================
String DccMsg;
String LastDccMsg;

void notifyDccAccTurnoutOutput( uint16_t Addr, uint8_t Direction, uint8_t OutputPower ) {
  Addr+=3;                                         // pour compenser le decodage d'adresse NMRA
  DccMsg=Addr;
  DccMsg+=",";
  DccMsg+=Direction;
  DccMsg+=",";
  DccMsg+=OutputPower;
  if (LastDccMsg!=DccMsg) {
    LastDccMsg=DccMsg;
#if defined(Debug)
      Serial.print(F("notifyDccAccTurnoutOutput: "));
      Serial.print(Addr,DEC);
      Serial.print("(+");
      Serial.print(AccOffset);
      Serial.print("),");
      Serial.print(Direction,DEC);
      Serial.print(',');
      Serial.println(OutputPower, HEX);
#endif
    tmpDccAddress=Addr;
    boolean enable=(Direction==1) ? 1: 0;
    int adrFonction=tmpDccAddress-DccAddress;
    // Serial.print(adrFonction);Serial.print("  enable : ");Serial.println(enable);

    switch (Mode) {
      case 0:   // Placer ici la recherche de gDcc correspondant
        DecodeDcc(tmpDccAddress+AccOffset,enable);
        if (adrFonction>=0 && adrFonction<MaxDccAddress) {
          id=0;
#if defined(MAM_PN)
          switch (adrFonction) {
            case 0 :
              if (enable) id = CV_BP_ACTION_1_0;  // Son CV 160
              else        id = CV_BP_ACTION_1_1;  // Son CV 150
              break;
            case 1 :
              if (enable) id = CV_BP_ACTION_2_0;  // Son CV 180
              else        id = CV_BP_ACTION_2_1;  // Son CV 170
              break;
            case 2 :
              if (enable) id = CV_BP_ACTION_3_0;  // Son CV 200
              else        id = CV_BP_ACTION_3_1;  // Son CV 190
              break;
            case 3 :
  #ifndef UseCommandeBouton
              id = enable ? PNoccupe: PNlibre;
              BP[4] = enable;
  #else
              if (enable) id = CV_ADKEY_MP3_1;    // Son CV 210
              else id=0;
  #endif
              break;
/*
            case 4 :
              if (enable) id = CV_ADKEY_MP3_9;    // Son CV 210
              else id=0;
              break;
            case 5 :
              if (enable) id = CV_ADKEY_MP3_10;   // Son CV 220
              else id=0;
              break;
            case 6 :
              if (enable) id = CV_ADKEY_MP3_11;   // Son CV 230
              else id=0;
              break;
            case 7 :
              if (enable) id = CV_ADKEY_MP3_12;   // Son CV 240
              else id=0;
              break;
*/
            default :
              id = 0;
              break;
          }
#elif defined(MAM_BP)
  #if defined(UseRedirections)
          if (CptTco<CptTcoLimit) id=Dcc.getCV(CV_Base_REDIR_AK+adrFonction)-1; // Redirection des AK vers les fonctions
  #elif defined(UseBasicRedir)
          if (CptTco<CptTcoLimit) id=Dcc.getCV(CV_Base_REDIR_BP+adrFonction)-1; // Affectation des premieres fonctions aux AK
  #else
          if (CptTco<CptTcoLimit) id=adrFonction;                               // Affectation des premieres fonctions aux AK
  #endif
          BP[id]=enable;
#endif
        }
        //Serial.println(id);
        //Serial.println(tmpDccAddress);
        break;
      case 1:                                      // Display received accessory address and state
        String msg="";
        tmpDccAddress=tmpDccAddress+AccOffset;
        msg = (const __FlashStringHelper *) ModeDetect;
        msg += " : ";
        msg += String(tmpDccAddress);
        msg += F(" Groupe : ");
        msg += String(tmpDccAddress/4);
        msg += F(" Item : ");
        msg += String((tmpDccAddress-1)%4+1);
        msg += F(" Dir : ");
        msg += String(Direction);
        msg += F(" Power : ");
        msg += String(OutputPower);
        Serial.println(msg);
        msg = (const __FlashStringHelper *) SaveDcc;  //Cmd_SaveDcc;
        msg += String(tmpDccAddress);
        Serial.println(msg);
        // lcdDisplay(lcd_Infol,0,"?"+String(tmpDccAddress)+"  ");
        break;
    }
  }
}

#endif // (USE_DCC)

// ========================================================
// Sauvegarde Adressse Dcc
// ========================================================
void SaveDccAddress(uint16_t DccAd, bool save) {
  String msg="";
  DccAddress=DccAd&0x3FC;                         // Enregistrement de l'adresse
  Sprint_F(KnownDcc);
  Serial.print(DccAddress);
  Serial.print("-");
  Serial.println(DccAddress + MaxDccAddress - 1);
  if (save) {
#ifdef USE_LCD
  msg=(const __FlashStringHelper *)SauveAdr;
  lcdDisplay(lcd_UsbRl,lcd_UsbRc,msg+String(DccAddress));
#endif
    Dcc.setCV(CV_ACCESSORY_DECODER_ADDRESS_MSB, (DccAddress>>8)&7);
    Dcc.setCV(CV_ACCESSORY_DECODER_ADDRESS_LSB, DccAddress&0xFC);
  }
}

// ========================================================
// GetCV
// ========================================================
void GetCV() {
  // lcdDisplay(lcd_Infol,lcd_Infoc,F("Lecture CV"));
  DccAddress=((Dcc.getCV(CV_ACCESSORY_DECODER_ADDRESS_MSB)&7)*256) + (Dcc.getCV(CV_ACCESSORY_DECODER_ADDRESS_LSB)&0xFC);
  // Initialisation à partir des CV
  Volume=Dcc.getCV(CV_VOLUME);                          // Volume Son MP3
  DelaiCassePieds = Dcc.getCV(CV_CASSE_PIEDS);          // Delai anti casse-pieds
  bMillisLimit = Dcc.getCV(CV_BLINK_LEDS)*100;          // Delai Clignotement LED
  for (uint8_t i=0; i<8; i++) {
    BPs[i].Type=Dcc.getCV(CV_Base_8Fct+10*i);
#if defined(Juse)
    if (BPs[i].Type!=3) BPs[i].Pin=JPins[Dcc.getCV(CV_Base_8Fct+10*i+1)-1]; // Connecteur J1 à J8
    else                BPs[i].Pin=Dcc.getCV(CV_Base_8Fct+10*i+1);          // MP3
#else
    BPs[i].Pin=Dcc.getCV(CV_Base_8Fct+10*i+1);
#endif
    BPs[i].Min=Dcc.getCV(CV_Base_8Fct+10*i+2);
    BPs[i].Max=Dcc.getCV(CV_Base_8Fct+10*i+3);
    BPs[i].Speed=Dcc.getCV(CV_Base_8Fct+10*i+4);
    BPs[i].Angle=(BPs[i].Min+BPs[i].Max)/2;
    BPs[i].Objectif=BPs[i].Angle;
  }
}

// ========================================================
// Make FactoryDefaultCVIndex non-zero and equal to num CV's to be reset
// to flag to the loop() function that a reset to Factory Defaults needs to be done
// ========================================================
void notifyCVResetFactoryDefault()
{
  FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs)/sizeof(CVPair);
};

// ========================================================
// recupere les CV par défaut
// ========================================================
void ResetCV() {
///  for (uint8_t j=CV_Base_Reset; j < MaxCV; j++ ) Dcc.setCV(j, 255);

  for (uint8_t j=0; j < sizeof(FactoryDefaultCVs)/sizeof(CVPair); j++ ) {
    Dcc.setCV( FactoryDefaultCVs[j].CV, FactoryDefaultCVs[j].Value);
//  Serial.print(FactoryDefaultCVs[j].CV);
//  Serial.print("   ");
//  Serial.println(FactoryDefaultCVs[j].Value);
  }
}
