/*==========================================================
Lcd_Std.h
============================================================*/

#include "Lcd.h"
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#if defined(Lcd1602x27) 
  LiquidCrystal_I2C lcd(0x27,16,2); // set the LCD address to 0x27 for a 16 chars and 2-line display
#elif defined(Lcd1602x21) 
  LiquidCrystal_I2C lcd(0x2F,16,2); // set the LCD address to 0x2F for a 16 chars and 2-line display
#elif defined(Lcd2004x27) 
  LiquidCrystal_I2C lcd(0x27,20,4); // set the LCD address to 0x27 for a 20 chars and 4-line display
#elif defined(Lcd2004x3F) 
  LiquidCrystal_I2C lcd(0x3F,20,4); // set the LCD address to 0x3F for a 20 chars and 4-line display
#endif

// =========================================================
// Init Lcd
// =========================================================
void InitLcd() {
#if defined(Lcd2004x27) || defined(Lcd2004x3F)
  lcd.begin();                    // initialize the lcd
#elif defined(Lcd1602x27) || defined(Lcd1602x2F)
  lcd.begin();                    // initialize the lcd
//  lcd.init();                     // initialize the lcd NOT IN LIB
#endif
  lcd.backlight();                // Retro eclairage
}

// =========================================================
// Line, Column, Message
// =========================================================
void lcdDisplay(uint8_t l, uint8_t c, String msg)
{
#ifdef USE_LCD
  lcd.setCursor(c,l);
  #if defined(LcdXB)
//  for (int i=0;i<msg.length();i++) {
  for (uint8_t i=0;i<msg.length();i++) {
    lcd.print(msg.substring(i,i+1));
  }    
  #else
  lcd.print(msg);
  #endif
#endif
}  

// =========================================================
// Scan I2C
// =========================================================
#if defined(I2C_Scan)
void ScanI2C() {
  byte error, address;
  uint8_t nDevices;

  Serial.println(F("Scanning..."));

  nDevices = 0;
  for(address = 1; address < 127; address++ ) 
  {
    // The i2c_scanner uses the return value of
    // the Write.endTransmisstion to see if
    // a device did acknowledge to the address.
    Wire.beginTransmission(address);
    error = Wire.endTransmission();

    if (error == 0)
    {
      Serial.print(F("I2C a l'adresse 0x"));
      if (address<16) 
        Serial.print("0");
      Serial.print(address,HEX);
      Serial.println("  !");

      nDevices++;
    }
    else if (error==4) 
    {
      Serial.print(F("Erreur inconnue adresse 0x"));
      if (address<16) 
        Serial.print("0");
      Serial.println(address,HEX);
    }    
  }
  if (nDevices == 0)
    Serial.println(F("Aucun I2C trouve"));
  else
    Serial.println(F("Fin du scan"));
}
#endif
