/*=========================================================
  -------------------------------------------------------------------------------------------------------
  USB:              contains methods to receive USB commands, print with memory saving
  DCC:      		    contains methods to read and interpret text commands from the serial/Ethernet line,
  LCD:              contains methods to print on I2C LCD display
  MP3:              contains methods to play Mp3 or Mp3 list with DfPlayer
  Servomoteurs:     contains methods to move servos with or without SlowMotionServo library

  Attention, l'ordre des include ne doit pas être modifié!
  -------------------------------------------------------------------------------------------------------
  ===========================================================*/

// ========================================================
// Configuration generale
// ========================================================

#define MAM_BP                // A ne pas oublier !

#define USE_USB               // Commandes USB
#define USE_DCC               // Commandes DCC
#define USE_BP                // Commandes BP (sauf Detect DCC)
#define USE_LCD               // Choisir son type de LDC :

#ifdef USE_LCD
// #define Lcd1602x27           // LCD 2 lignes de 16 caracteres, adresse 0x27
// #define Lcd1602x2F           // LCD 2 lignes de 16 caracteres, adresse 0x2F
// #define Lcd2004x27           // LCD 4 lignes de 20 caracteres, adresse 0x27
#define Lcd2004x3F            // LCD 4 lignes de 20 caracteres, adresse 0x2F
#define I2C_Scan              // Le scan I2C est disponible
#endif
#define USE_MP3
#define USE_Servo

/* ========================================================
    Resultat des differentes compilations
   ========================================================
  SlowMotionSrv+Sans Redirection
  Le croquis utilise 30662 octets (99%) de l'espace de stockage de programmes. Le maximum est de 30720 octets.
  Les variables globales utilisent 1504 octets (73%) de mémoire dynamique, ce qui laisse 544 octets pour les variables locales. Le maximum est de 2048 octets.

  SlowMotionSrv+UseRedirections
  Le croquis utilise 27866 octets (90%) de l'espace de stockage de programmes. Le maximum est de 30720 octets.
  Les variables globales utilisent 1494 octets (72%) de mémoire dynamique, ce qui laisse 554 octets pour les variables locales. Le maximum est de 2048 octets.

  SlowMotionSrv+UseRedirections
  Le croquis utilise 30700 octets (99%) de l'espace de stockage de programmes. Le maximum est de 30720 octets.
  Les variables globales utilisent 1544 octets (75%) de mémoire dynamique, ce qui laisse 504 octets pour les variables locales. Le maximum est de 2048 octets.

  Servo+UseRedirections (sans SoftPWM)
  Le croquis utilise 24540 octets (79%) de l'espace de stockage de programmes. Le maximum est de 30720 octets.
  Les variables globales utilisent 1210 octets (59%) de mémoire dynamique, ce qui laisse 838 octets pour les variables locales. Le maximum est de 2048 octets.

  Servo sans UseRedirections (avec SoftPWM)
  Le croquis utilise 28488 octets (92%) de l'espace de stockage de programmes. Le maximum est de 30720 octets.
  Les variables globales utilisent 1383 octets (67%) de mémoire dynamique, ce qui laisse 665 octets pour les variables locales. Le maximum est de 2048 octets.

*/
// ========================================================
// Options de compilation
// ========================================================
//#define Debug                 // affichages complementaires pour le debug
#define AideUSB               // Affichage USB
#define CvInitTrigger         // Init des CV apres modification CV
//#define UseCassePiedsLock     // Anti Casses Pieds
#define UseRedirections       // Redirection individuelle des BP et AK par CV
//#define UseBasicRedir         // Redirection globale des BP et AK par CV

//#define SlowMotionSrv

// Utilisation de la bibliotheque SlowMotionServo
#if defined(SlowMotionSrv)    // Utilisation de la bibliotheque SlowMotionServo
//#define Servo_Linear
#define Servo_Smooth
#else
#include "SoftPWM.h"
#endif

#define Juse
//#define NOTIFY_DCC_MSG      // unprint/print all DCC Packets
//#define All_cv              // Affiche les CV à 255

// ========================================================
//  Inclusion area
// ========================================================
// Buffer circulaire:  https://github.com/Locoduino/RingBuffer/releases

#include "ComVar.h"

#if defined(USE_USB)
#include "Usb_Std.h"
#endif

#if defined(USE_DCC)
#include "Dcc_Std.h"
#endif

#if defined(USE_LCD)
#include "Lcd_Std.h"
#endif

#if defined(USE_MP3)
#include "Mp3_Std.h"
#endif

#if defined(USE_Servo)
#include "Servos_Std.h"
#endif
