/* ========================================================
 * MAM BP Version 4.3
 *
 * X Bouillard : 2020/01/03 creation
 * P Chavatte  : 2020/04/01 test, debug
 * Mise à jour : 2021/01/19
 * 
 * librairie NMRADcc
 * https://www.arduinolibraries.info/libraries/nmra-dcc
 *
 * Détection automatique Adresse DCC (confirmation par ! ou BP1) (ex SW2)
 * CV par défaut si EEPRROM vierge (CV29=255)
 *
 * Commandes DCC, USB, BP, interrupteurs, BP et occupation 4 cantons
 *
 * Parametrage par CV     Numero
 *  Volume                : 15
 *  Anti casse pieds      : 19
 *  Redirections BP et AK : 30 à 49
 *  BP/ DCC(1-8)          : 50 à 129   actions sur les pins J1..8
 *      DCC(9-16)         : 150 à 229  MP3
 *
 *  adresses DCC 1 à 8   = AK boutons Ad_key pour les fonctions (Led avec extinction temporisées ou inversion On/Off, Servomoteurs Mini Maxi, Sons unitaire)
 *  adresses DCC 9 à 12  = AK boutons Ad_key réservées aux listes de sons (Type annonce)
 *  adresses DCC 13 à 16 = BP1 à BP4, réservées aux listes de sons (Type annonce)
 *  BP1 A0 = détection / Sauvegarde adresse DCC (appui long)              (marqué SW2 sur CI MAM1)
 *  BP2 A1 = On/Off (autorisation des commandes par BP appui tres long)   mini clavier bouton de gauche (+ eloigne du connecteur)
 *  BP3 A2 =                                                              mini clavier bouton du milieu (intermediaire)
 *  BP4 Occ A7 =                                                          mini clavier bouton de droite (- eloigne du connecteur)

Servo+UseRedirections+SlowMotionSrv sans SoftPWM
Le croquis utilise 30228 octets (98%) de l'espace de stockage de programmes. Le maximum est de 30720 octets.
Les variables globales utilisent 1545 octets (75%) de mémoire dynamique, ce qui laisse 503 octets pour les variables locales. Le maximum est de 2048 octets.

Servo+UseRedirections+SoftPWM sans SlowMotionSrv
Le croquis utilise 28426 octets (92%) de l'espace de stockage de programmes. Le maximum est de 30720 octets.
Les variables globales utilisent 1460 octets (71%) de mémoire dynamique, ce qui laisse 588 octets pour les variables locales. Le maximum est de 2048 octets.

=========================================================== */

// ========================================================
const char Msg_Decoder[]       PROGMEM = {"MAM-BP  "};  // Animation par Bouton Poussoir
const char Msg_Vrs[]           PROGMEM = {"4.3 "};
// ========================================================
uint8_t MAM_Version=43;

// ========================================================
// Connecteurs J/Pin Arduino
// ========================================================
uint8_t JPin(uint8_t D) {
  if (D<7) D=D-2;
  else D=D-4;
  return D;
}

#include "MAM-BP.h"

// ========================================================
// Display Parameters
// ========================================================
void List() {
  Sprintln_F(HlpSep);
  for (uint8_t i=0;i<8;i++) {
    uint8_t j = BPs[i].Pin;
    Serial.print(i+OffsetFct);
    if(i<9) Serial.print(" ");
    Sprint_F(CvDef0);
    if ((CV_Base_8Fct+10*i)<100) Serial.print("0");
    Serial.print((CV_Base_8Fct+10*i)/10);
///    Serial.print(i);
    Serial.print("x");
    if (BPs[i].Type==TypeFixLed || BPs[i].Type==TypeBlinkLed || BPs[i].Type==TypeFading) {
      if (BPs[i].Type==TypeFixLed)      Sprint_F(CvType0);
      if (BPs[i].Type==TypeFading)      Sprint_F(CvType2);
      else                              Sprint_F(CvType1);
      Serial.print(JPin(BPs[i].Pin));
      Serial.print("=");
      if(j<10) Serial.print(" ");
      Serial.print(j);
      if (BPs[i].Min!=BPs[i].Max) {
        Sprint_F(CvDef1);
        Serial.print(BPs[i].Min);
        Serial.print("-");
        Serial.print(BPs[i].Max);
        Sprint_F(CvDef3);
      }
      if (BPs[i].Speed>0) {
        Sprint_F(Msg_Tempo);
        Serial.print(BPs[i].Speed);
        Serial.print("s");
      }
      Serial.println();
    }
    else if (BPs[i].Type==Typesolenoid) {
      Sprint_F(CvType5);
      Serial.print(JPin(BPs[i].Pin));
      Serial.print("=");
      if(j<10) Serial.print(" ");
      Serial.print(j);
      if (BPs[i].Speed>0) {
        Sprint_F(Msg_Tempo);                                       // par pas de 1/10 s
        Serial.print((float)BPs[i].Speed/10);
        Serial.print("s");
      }
      Serial.println();
    }
    else if (BPs[i].Type==TypeMP3List) {
      Sprint_F(CvType3);
      Serial.println(BPs[i].Pin);
    }
    else if (BPs[i].Type==TypeServo) {
      Sprint_F(CvType4);
      Serial.print(JPin(BPs[i].Pin));
      Serial.print("=");
      if(j<10) Serial.print(" ");
      Serial.print(j);
      Sprint_F(CvDef1);
      Serial.print(BPs[i].Min);
      Serial.print("-");
      Serial.print(BPs[i].Max);
      Sprint_F(CvDef3);
      Sprint_F(CvVitesse);
      Serial.println(BPs[i].Speed);
    }
    else {
      Sprint_F(CvUndef1);
      Serial.print(CV_Base_8Fct+10*i);
      Sprintln_F(CvUndef2);
    }
  }

  for (uint8_t ik=8;ik<16;ik++) {
    Serial.print(ik+OffsetFct);
    if(ik<9) Serial.print(" ");
    Sprint_F(CvDef0);
    Serial.print((CV_Base_MP3+10*(ik-8))/10);
    Serial.print("x");
    Sprint_F(CvMp3);
    String L;
    for (uint8_t i=0;i<9;i++) {
      uint8_t tmp=Dcc.getCV(CV_Base_MP3+10*(ik-8)+i);
      if (tmp!=255) L+="_"+String(tmp);
    }
    Serial.println(L.substring(1));
  }
}

// ========================================================
// Execute les commandes recueillies
// ========================================================
void DoAction() {
  if (id>-1) {
    Sprint_F(Msg_Fct);
    Serial.print(id+1);
    if (id<8) {
      Serial.print(F(" Type"));
      Serial.print(BPs[id].Type);
      Serial.print(F(" Pin"));
      Serial.print(BPs[id].Pin);
    }
    Serial.print(" ");
    Serial.println(BP[id]);
  #ifdef USE_LCD
    lcdDisplay(lcd_Infol,lcd_Infoc,(const __FlashStringHelper *)Clear_Lcd);
    if (BP[id]) lcdDisplay(lcd_Infol,lcd_Infoc,String(id+1)+ F(" On  " ));
    else        lcdDisplay(lcd_Infol,lcd_Infoc,String(id+1)+ F(" Off " ));
  #endif
    if (id>=8) {                                                    // fonctions MP3 de 9 à 20
      if (id==16) {                                                 // ?
        Serial.println(F("DCC16"));

      } else if (id==17) {                                          // ?
        Serial.println(F("DCC17"));

      } else if (id==18) {                                          // ?
        Serial.println(F("DCC18"));

      } else if (id==19) {                                          // Reglage du volume
        if (BP[id]) {                                               // Volume +
          if (Volume<30) Volume++;
        } else {                                                    // Volume -
          if (Volume>0)  Volume--;
        }
        Sprint_F(Msg_Volume);
        Serial.println(Volume);
        mp3_set_volume(Volume);
      } else {                                                      // Joue une liste de sons
        String L;
        if (BP[id]) {
          for (uint8_t i=0;i<10;i++) {
            PL[i]=Dcc.getCV(CV_Base_MP3+10*(id-8)+i);
            if (PL[i]!=255) L+="_"+ String(PL[i]);
          }
          PLi=0;
          Serial.println(L.substring(1));
        } else {
          for (uint8_t i=0;i<10;i++) {
            PL[i]=255;
          }
        }
        //Play_List="J1_3_97_4_76_5_51_6_";
      }                                                             // fonctions bouton de 1 à 8
    } else if (BPs[id].Type==TypeFixLed                             // 0=LED fixe
            || BPs[id].Type==TypeBlinkLed                           // 1=LED clignotante
            || BPs[id].Type==TypeFading) {                          // 2=LED fading
      Serial.print(F(" LED J"));                                    // Type             CV 50+10*id     TypeFixLed ou TypeBlinkLed
      Serial.print(id+1);
      Serial.print(F(" type "));
      Serial.print(BPs[id].Type);
      if (!BP[id]) Tp[id]=BPs[id].Speed;                            // Tempo            CV 54+10*id     Temporisation avant extinction ou niveau bas
      else Tp[id]=0;
      Sprint_F(Msg_Tempo);
      Serial.println(Tp[id]);
  #ifdef USE_LCD
      lcdDisplay(lcd_Infol,7,"T"+String(Tp[id]));                   // affichage Tempo
  #endif                                                            // SlowMotionSrv
      uint8_t Nr=BPs[id].Pin; //Dcc.getCV(CV_Base_8Fct+10+id);      // Pin              CV 51+10*id
      pinMode(Nr, OUTPUT);
      delay(10);
      if (BPs[id].Type==0) Sprint_F(CvType0);
      else if (BPs[id].Type==2) Sprint_F(CvType2);
           else                 Sprint_F(CvType1);
      Serial.print(String(JPin(Nr)) + "=" + String(Nr) + " => " );
#ifdef SlowMotionSrv                                                // SlowMotionSrv
      if (BPs[id].Max==BPs[id].Min || (Nr!=4 && Nr!=11)) {          // PWM si on à Min <> Max et PWM autorisé (ni J2=4 ni J7=11), On/Off sinon
#else
      if (BPs[id].Max==BPs[id].Min) {                               // PWM si on à Min <> Max
#endif
        Serial.print(CV_Base_8Fct+10*id+2);Serial.print("   "); Serial.println(CV_Base_8Fct+10*id+3); 
        Sprint_F(Msg_Digital);
        if (BP[id])         BPs[id].Angle=1;
        else if(Tp[id]==0)  BPs[id].Angle=0;
        Serial.println(BP[id]);
        if(Tp[id]==0) digitalWrite(Nr,BP[id]);                      // Digital Led
      } else {
        Sprint_F(Msg_Analog);
        if (BP[id] || Tp[id]) {
#ifndef SlowMotionSrv                                                // SlowMotionSrv
          SoftPWMSetFadeTime(JPins[id], 100*Dcc.getCV(CV_Base_8Fct+10*id+2), 100*Dcc.getCV(CV_Base_8Fct+10*id+3));
#endif
          Serial.println(BPs[id].Max);
          BPs[id].Angle=BPs[id].Max;
        } else {
#ifndef SlowMotionSrv                                                // SlowMotionSrv
           SoftPWMSetFadeTime(JPins[id], 100*Dcc.getCV(CV_Base_8Fct+10*id+3), 100*Dcc.getCV(CV_Base_8Fct+10*id+2));
#endif
          Serial.println(BPs[id].Min);
          BPs[id].Angle=BPs[id].Min;
///        if(Tp[id]==0) BPs[id].Angle=BPs[id].Min;
        }
#ifdef SlowMotionSrv
        if(Tp[id]==0)   analogWrite(Nr,BPs[id].Angle);              // Analog Led Min   CV 52+10*id   ou Led Max    CV 53+10*i
#else
        if(Tp[id]==0)   SoftPWMSet(Nr,BPs[id].Angle);               // Analog Led Min   CV 52+10*id   ou Led Max    CV 53+10*i
#endif                                                              // SlowMotionSrv
      }
      if(Tp[id]!=0) {
        Sprint_F(Msg_Tempo);
        Serial.print(Tp[id]);
        Serial.println(" s");
      }
    } else if (BPs[id].Type==Typesolenoid) {                        // 5=aiguillage à solenoide
      uint8_t Nr=BPs[id].Pin; //Dcc.getCV(CV_Base_8Fct+10+id);      // Pin              CV 51+10*id
      if (BP[id]) Nr++;
      pinMode(Nr, OUTPUT);
      delay(10);
      Serial.print(F(" Solenoid "));
      if (BP[id]) Serial.print(id+1);
      else Serial.print(id);
      Serial.print(F(" type "));
      Serial.print(BPs[id].Type);
      if (!BP[id]) Tp[id]=BPs[id].Speed;                            // Tempo            CV 54+10*id     Activation temporisée à 1 puis 0
      else Tp[id]=0;
      Sprint_F(Msg_Tempo);
      Serial.println(Tp[id]);
      if (BPs[id].Type==2) Sprint_F(CvType2);
      Serial.print(String(JPin(Nr)) + "=" + String(Nr) + " =>" );
      Sprint_F(Msg_Digital);
      digitalWrite(Nr,1);                                           // activation solenoide
      Serial.println(BP[id]);
      if (id<8) {
        Tp[id] = Dcc.getCV(3+id/2);
        while(Tp[id] > 0) {
          delay(100);
          Tp[id]--;
          BP[id]=0;
          //Serial.println(Tp[id]);
        }
        //Serial.print(Nr);Serial.print("   ");Serial.print(id);Serial.print("   ");Serial.println(Tp[id]);
      }
      digitalWrite(Nr,BP[id]);                                      // desactivation solenoide si Tempo > 0
    } else if (BPs[id].Type==TypeMP3List) {                         // 3=MP3           CV 150+10*id
      uint8_t Nr=BPs[id].Pin; //Dcc.getCV(CV_Base_8Fct+10+id);      // MP3             CV 151+10*id
      if (BP[id]) {
  #ifdef USE_LCD
        lcdDisplay(lcd_SndNrl,lcd_SndNrc,String(Nr));
  #endif
        Sprintln_F(Msg_Joue);
        Track_Play(Nr,"");
      } else {
        Sprint_F(Msg_Stop);
        Serial.print(Nr);
        mp3_stop();
      }
    } else if (BPs[id].Type==TypeServo) {                           // 4=servo         CV 50+10*id
      uint8_t Nr=BPs[id].Pin; //Dcc.getCV(CV_Base_8Fct+10+id);      // Pin             CV 51+10*id
      Serial.print(" "); Sprint_F(Msg_Servo); Serial.print(" ");
      Serial.println(String(JPin(Nr)) + "=" + String(Nr) + " =>" + BP[id]);
#if defined(SlowMotionSrv)                            // >>>>> Utilisation de la bibliotheque SlowMotionServo
      if (BP[id]) BPs[id].myServo.goTo(0.0);          // positionne le servo Min Servo
      else        BPs[id].myServo.goTo(1.0);          // positionne le servo Max Servo
      delay(200);
#else                                                 // >>>>> Utilisation de la bibliotheque Servo
      if (BP[id])                                     // positionne le servo Max Servo  CV 54+10*id
        BPs[id].Objectif=BPs[id].Max;
      else                                            // positionne le servo Min Servo  CV 53+10*id
        BPs[id].Objectif=BPs[id].Min;
#endif                                                // SlowMotionSrv
    } else {
      Sprintln_F(UnKnown);
    }
  }
  id=-1;                                              // Reset commande apres execution de la commande!
}

// ========================================================
// Redirection des AK et BP
// ========================================================
#if defined(UseRedirections)
void ActivateId(int Nr) {
  if (BP_actif) {
    Serial.print(CptTco);
    id=Dcc.getCV(Nr)-1;
    if (id>=MaxDccAddress) {
      id=-1;
      Sprintln_F(UnKnown);                            // Redirection impossible
    }
    else {
      BP[id]=(CptTco<CptTcoLimit);                    // Affectation des dernieres fonctions aux BP
#if defined(UseCassePiedsLock)   // Anti Casses Pieds
      CptCassePieds=DelaiCassePieds;
#else
      CptCassePieds=0;
#endif
    }
  } else Sprintln_F(BPOff);                           // BP Inactifs
}
#endif

// ======================================================== 
// Etat des BP
// ======================================================== 
void EtatBP() {
   Sprint_F(Msg_BPactif);
   if (BP_actif) {
     Serial.println("On");
  #ifdef USE_LCD
     lcdDisplay(lcd_BPAdl,lcd_BPAdc,"On ");
  #endif
   } else {
     Serial.println("Off");
  #ifdef USE_LCD
     lcdDisplay(lcd_BPAdl,lcd_BPAdc,"Off");
  #endif
   }
}

// ========================================================
// Gestion des commandes recues
// ========================================================
#if defined(USE_USB)              // Commandes USB
void Commandes(String inStr) {
  Serial.println();
  Sprint_F(Msg_Recu_USB);
  Serial.println(inStr);
#ifdef USE_LCD
    lcdDisplay(lcd_UsbRl,lcd_UsbRc,F("                    "));
    lcdDisplay(lcd_UsbRl,lcd_UsbRc,inStr);
    lcdDisplay(lcd_UsbRl,lcd_UsbRc+inStr.length()-1," ");
#endif
  String c = String(inStr.charAt(0));   // isolation de la commande
  c.toUpperCase();
  String Prm = inStr.substring(1);      // isolation des paramètres
  Prm.replace("  "," ");
  if (inStr.charAt(1)==' ') Prm = inStr.substring(2);
  uint8_t i=0;
  Prm=Prm+" ";

  if (Prm.indexOf(' ')==0) Prm=Prm.substring(1);
  Prm.toUpperCase();
  int p=Prm.indexOf(' ');
  while (p>0) {
    Param[i]=Prm.substring(0,p);
    Prm=Prm.substring(p+1);
    p=Prm.indexOf(' ');
    i++;
  }
  // ============================== Commandes génériques
  if (c==Cmd_Help) {              // ...................... Help (Aide)
    SketchId();
    Help();
  } else if (c==Cmd_List) {       // ...................... List Parameters
    List();
  } else if (c==Cmd_Work) {       // ...................... On /Off des BP
    if (Param[0].indexOf("0")==0)       BP_actif=false;
    else if (Param[0].indexOf("1")==0)  BP_actif=true;
    else BP_actif=!BP_actif;
    EtatBP();
  } else if (c==Cmd_Volume) {     // ...................... Reglage du Volume
    if (inStr.length()>2) {
      Volume=Param[0].toInt();
      mp3_set_volume(Volume);
    }
    Sprint_F(Msg_Volume);
    Serial.println(Volume);
  } else if (c==Cmd_Joue) {       // ...................... Joue un MP3 ou une liste
    Sprint_F(Msg_Joue);
    Serial.println(Param[0]);
    if (Param[0].indexOf('_')>0) {
      Param[0]+="_";
      uint8_t i;
      for (i=0;i<9;i++) PL[i]=255;
      i=0;
      while (Param[0].indexOf('_')>0) {
        PL[i]= (Param[0].substring(0,Param[0].indexOf('_'))).toInt();
        Param[0]=Param[0].substring(Param[0].indexOf('_')+1);
        i++;
      }
      PLi=0;
    }
    else
      mp3_play(Param[0].toInt());
    //Volume=Param[1].toInt();
    //mp3_set_volume(Volume);
  } else if (c==Cmd_Stop) {       // ...................... Arrêt du MP3
    Sprintln_F(Msg_Stop);
    mp3_stop();
  } else if (c==Cmd_Servo) {      // ...................... Positionne un servo
    Sprintln_F(Msg_Servo);
    UpdateServo(Param[0].toInt()-1,Param[1].toInt());
  } else if (c==Cmd_Test) {       // ...................... Teste une fonction
    Sprint_F(Msg_Test);
    Serial.print(Param[0].toInt());
    Serial.print(" ");
    Serial.println(Param[1]);
    id=Param[0].toInt()-OffsetFct;
    if (Param[1][0]=='O' || Param[1][0]=='1') BP[id]=true;
    else if (Param[1][0]=='F' || Param[1][0]=='0') BP[id]=false;
    else {
      Sprint_F(Refus);
      Sprint_F(Msg_Attendu);
    }

#if defined(Debug)
  } else if (c==Cmd_SetDCC) {     // ...................... Set adresse DCC
    if (Param[1][0]=='F' || Param[1][0]=='0')      SaveDccAddress(Param[0].toInt(), false); // Show DCC
    else if (Param[1][0]=='O' || Param[1][0]=='1') SaveDccAddress(Param[0].toInt(), true);  // Set DCC
    else {
      Sprint_F(Refus);
      Sprint_F(Msg_Attendu);
    }
  } else if (c==Cmd_DCC) {        // ...................... Teste une commande DCC
    Sprintln_F(Msg_SimuDCC);
    Serial.print(Param[0]);
    Serial.print(" ");
    Serial.print(Param[1]);
    if (Param[1][0]=='F' || Param[1][0]=='0')      DecodeDcc(Param[0].toInt(),false);      // DCC On
    else if (Param[1][0]=='O' || Param[1][0]=='1') DecodeDcc(Param[0].toInt(),true);       // DCC Off
    else {
      Sprint_F(Refus);
      Sprint_F(Msg_Attendu);
    }
  } else if (c==Cmd_AdKey) {      // ...................... Teste un BP
    id=Dcc.getCV(CV_Base_REDIR_AK+Param[0].toInt()-1)-1;      // A modifier suivant ce que l'on souhaite lancer
    Serial.print("=>");
    Serial.println(id);
    if (Param[1][0]=='O' || Param[1][0]=='1') BP[id]=true;    // Affectation des dernieres fonctions aux BP
    else if (Param[1][0]=='F' || Param[1][0]=='0') BP[id]=false;
    else {
      Sprint_F(Refus);
      Sprint_F(Msg_Attendu);
    }
#endif // Debug
  } else if ((c==Cmd_SetCV) || (c==Cmd_SetCVbis)) {      // ...................... Ecriture CV
    if (Param[1].toInt()>255) {
      Sprint_F(Refus);
      Sprintln_F(Msg_BadCV);
    } else {
    Sprintln_F(Msg_SetCV);
      Dcc.setCV(Param[0].toInt(),Param[1].toInt());
      Serial.println("CV "+ Param[0]+ " = " + String(Dcc.getCV(Param[0].toInt())));
#if defined(CvInitTrigger)        // Init des CV apres modification CV
      GetCV();
//      List();
#if defined(SlowMotionSrv)
      InitServo();
#endif // SlowMotionSrv
#endif // CvInitTrigger
    }
  } else if ((c==Cmd_GetCV) || (c==Cmd_GetCVbis)) {      // ...................... Lecture CV
    Sprintln_F(Msg_GetCV);
    if (Param[0][0]=='\r') {
      for (uint8_t i=0;i<MaxCV;i++) {
        if (i%10 == 0) Serial.print("< ");
        String v=String(i);
        while (v.length()<3) v = " " + v;
        v += "=" + String(Dcc.getCV(i));
        if (i%10 != 9) while (v.length()<10) v += " ";
        Serial.print(v);
        if (i%10 == 9) Serial.println(" >");
      }
//      if (MaxCV%10 != 9) Serial.println(">");
    } else Serial.println("CV " + Param[0] + " = " + String(Dcc.getCV(Param[0].toInt())));
  } else if (c==Cmd_ResetCV) {    // ...................... Reset de CV a leur valeur par defaut
    Sprintln_F(Msg_ResetCV);
    ResetCV();
    Serial.print(F("Reset CV done"));
  } else if (c==Cmd_GetDcc) {     // ...................... Reconnaissance Adresse Dcc
    if (Mode==0) {
      Mode = 1;
    } else {
      Mode =0;
      Serial.print(F("Fin "));
    }
    Sprintln_F(Msg_GetDcc);
  } else if (c==Cmd_SaveDcc) {    // ...................... Sauvegarde Adresse Dcc
    tmpDccAddress=Param[0].toInt();
    SaveDccAddress(tmpDccAddress,true);
    delay(10);
    setup();                      // Reboot du module
#if defined(I2C_Scan)
  } else if (c==Cmd_ScanI2C) {    // ...................... Scan I2C
      Sprint_F(Msg_ScanI2C);
      ScanI2C();
#endif
  } else if (c==Cmd_Id) {         // ...................... Id Sketch
    SketchId();
  // ============================== Fin des commandes specifiques au sketch
  } else Sprintln_F(UnKnown);
}
#endif // USE_USB

// ========================================================
// Gestion des casse-pieds
// ========================================================
void ManageCassePieds() {
#if defined(UseCassePiedsLock)   // Anti Casses-Pieds
  currentMillis = millis();
  if (currentMillis - pCpMillis >= pCpMillisLimit) {
    pCpMillis = currentMillis;
    if (CptCassePieds>0) {
      CptCassePieds--;
      Serial.print(CptCassePieds+String(" "));
  #ifdef USE_LCD
      lcdDisplay(lcd_rbtKPl,lcd_rbtKPc,(String(CptCassePieds)+"   ").substring(0,3));
  #endif
    } else {
  #ifdef USE_LCD
      lcdDisplay(lcd_rbtKPl,lcd_rbtKPc-1,F(" *  "));
  #endif
    }
  }
#else 
  CptCassePieds=0;
#endif
}

// ========================================================
// Clignotement des LED TypeBlinkLed
// ========================================================
void ManageBlink() {
  if (currentMillis - bMillis >= bMillisLimit) {
    bMillis = currentMillis;
    BlinkLed = !BlinkLed;
    for (uint8_t n=0; n<8; n++) {
      if (BPs[n].Type==TypeBlinkLed) {
        if (BlinkLed) {
          if (BP[n] || Tp[n]!=0) {
#if defined(SlowMotionSrv)
          if (BPs[n].Max==BPs[n].Min || BPs[n].Pin==4 || BPs[n].Pin==12) {  // Ancien TypeDigLed PWM si ON à Min <> Max et PWM autorisé (ni J2=4 ni J8=12), On/Off sinon
#else
            if (BPs[n].Max==BPs[n].Min) {                           // Ancien TypeDigLed  PWM si on à Min <> Max
#endif
              digitalWrite(BPs[n].Pin,HIGH);                        // Digital Led
            } else {
#if defined(SlowMotionSrv)
              analogWrite(BPs[n].Pin,BPs[n].Max);                   // Analog Led Max   CV 53+10*n
#else
              SoftPWMSet(BPs[n].Pin,BPs[n].Max);                    // Analog Led Min   CV 53+10*n ou Led Max    CV 53+10*n
#endif
            }
          }
        } else {
#if defined(SlowMotionSrv)
        if (BPs[n].Max==BPs[n].Min || BPs[n].Pin==4 || BPs[n].Pin==12) {  // Ancien TypeDigLed  PWM si on à Min <> Max et PWM autorisé (ni J2=4 ni J8=12), On/Off sinon
#else
          if (BPs[n].Max==BPs[n].Min) {                             // Ancien TypeDigLed  PWM si on à Min <> Max
#endif
            digitalWrite(BPs[n].Pin,LOW);                           // Digital Led
          } else {
#if defined(SlowMotionSrv)
            analogWrite(BPs[n].Pin,BPs[n].Min);                     // Analog Led Min   CV 52+10*n
#else
            SoftPWMSet(BPs[n].Pin,BPs[n].Min);                      // Analog Led Min   CV 52+10*n ou Led Max    CV 53+10*n
#endif
          }
        }
      }
    }
  }
}

// ========================================================
// Verification fin MP3
// ========================================================
void ManageMP3() {
  if (currentMillis - mMillis >= mMillisLimit) {
    mMillis = currentMillis;
    if (Mp3Playing) {
      if (!digitalRead(BusyDfPlayer) && Cpt>0) Cpt--;               // Attente de la fin du message courant
      else {
        if (Cpt==0) Sprintln_F(MissMp3);
        else Sprintln_F(DoneMp3);
        Mp3Playing=false;
      }
    }
    else
    { if (PLi<10) {                                                 // Passage au MP3 suivant
        if (PL[PLi]!=255) {
          Track_Play(PL[PLi], String(PLi));
          if (PLi<10) PLi++;
        }
      }
    }
  }
}

// ========================================================
// Initialisation
// ========================================================
void setup() {
  // Initialisation USB
  Serial.begin(115200);
  Serial.println();

  // Affichage Nom sketch et version
  Sprintln_F(HlpSep);
  SketchId();

  // Initialisation LED pin 13
  pinMode(LED_BUILTIN, OUTPUT);

  // Initialisation BP
  pinMode(BP1Pin, INPUT_PULLUP);
  pinMode(BP2Pin, INPUT_PULLUP);
  pinMode(BP3Pin, INPUT_PULLUP);
  pinMode(OccPin, INPUT);
  pinMode(AdKeyPin, INPUT);

  // Lecture des CV
  GetCV();                                              // Lire les CV et récupérer l'adresse DCC du module
  if (Dcc.getCV(29)==255) {
    ResetCV();                                          // Valeurs par defaut
  }

  // Initialisation Lcd
#ifdef USE_LCD
  InitLcd();
  lcdDisplay(lcd_Idl,lcd_Idc,(const __FlashStringHelper *) Msg_Decoder);
  lcdDisplay(lcd_Idl,7,(const __FlashStringHelper *) Msg_Vrs);
//  lcdDisplay(lcd_Idl,11,F("DCC "));
  lcdDisplay(lcd_DccAdl,lcd_DccAdc,String(DccAddress)+"    ");  // Afficher l'adresse DCC du module
  lcdDisplay(lcd_Infol,lcd_Infoc,F("                "));
#endif

  SaveDccAddress(DccAddress,false);                     // Afficher l'adresse DCC du module
  tmpDccAddress=DccAddress;
  setupDCCDecoder();

// ========================================================
  // Initialisation à partir des CV
// ========================================================

//  Volume=Dcc.getCV(CV_VOLUME);                          // Volume Son MP3
  Attenuation=Dcc.getCV(CV_ATTENUATION);                // Volume Sonnerie attenuée
  
#if defined(SlowMotionSrv)
  InitServo();
  Sprintln_F(Msg_SlowMotion);
#else
  SoftPWMBegin();
  SoftPWMSet(LED_BUILTIN,0);
  // fade-up et fade-down pour la led LED_BUILTIN 13
  SoftPWMSetFadeTime(LED_BUILTIN, 500, 500);
  Serial.println(F("Servo & SoftPWM"));
#endif

#if defined(AideUSB)
  List();
  Help();
#endif

#ifndef CommandeBouton
  BP_actif=true;
#endif
  EtatBP();
  id=0;

  // Initialisation du DFPlayer
  InitMP3();
  mp3_set_volume(Volume);
  Track_Play(1,"");                  // jingle
  Serial.println();
}

// ========================================================
// Main loop
// ========================================================
void loop() {
  id = -1; 
  // Actualisations des commandes DCC : ===========================================================
  Dcc.process();
  // Reception des commandes: =====================================================================
  //  Commandes BlueTooth:

  //  Commandes I2C:

  //  Commandes CAN:

  //  Commandes Occupation, BP et interrupteurs (Ad_Key) ==========================================
  if ((CptCassePieds<=0) && (digitalRead(BP1Pin)==LOW)) { // ======================================
    Serial.print("BP1");
  #ifdef USE_LCD
    lcdDisplay(lcd_Infol,lcd_Infoc,(const __FlashStringHelper *)Clear_Lcd);
    lcdDisplay(lcd_Infol,lcd_Infoc,String("BP1"));
  #endif
    CptTco=0;
    while (not digitalRead(BP1Pin)==HIGH) {                         // Appui long sur BP1 pour sauvegarder apres detection
      if (CptTco<255) CptTco++;
      if (CptTco%10==0) Serial.print(CptTco);Serial.print(" ");
      if (CptTco==CptOnOffLimit) {                                  // On/Off des BP
        Sprintln_F(SauveDcc);
        SaveDccAddress(tmpDccAddress,true);                         // Enregistrement de l'adresse (appui long sur BP1)
  #ifdef USE_LCD
        lcdDisplay(lcd_DccAdl,lcd_DccAdc,String(tmpDccAddress)+"   ");
  #endif
        setup();                                                    // Reboot du module
      }
      if (CptTco==CptTcoLimit) {
  #ifdef USE_LCD
       lcdDisplay(lcd_Infol,lcd_DccAdc,"* ");
  #endif
      }
      delay(50);
    }
    Serial.println("-BP1");
    if (CptTco<CptOnOffLimit) {
      if (Mode==0) {                                                // Passage en mode DetectDCC()
        Mode=1;
        Sprintln_F(ModeDetect);
  #ifdef USE_LCD
        lcdDisplay(lcd_Infol,lcd_Infoc,"Detecte "+String(DccAddress)+"   ");
  #endif
        tmpDccAddress=DccAddress;
      } else {
        Mode=0;                                                     // Retour en mode normal
        Sprintln_F(ModeDecode);
  #ifdef USE_LCD
        lcdDisplay(lcd_Infol,lcd_Infoc,"Decode "+String(DccAddress)+"   ");
  #endif
      }
    }
      
#if defined(UseRedirections)
      ActivateId(CV_Base_REDIR_BP);                                 // Redirection des BP vers les fonctions
#elif defined(UseBasicRedir)
      if (BP_actif) {
        BP[Dcc.getCV(CV_Base_REDIR_BP)]=(CptTco<CptTcoLimit);       // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#else
      if (BP_actif) {
        BP[8]=(CptTco<CptTcoLimit);                                 // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#endif
#if defined(USE_BP)                                                 // Commandes BP
  } else if (CptCassePieds==0) {                                    // BP_actif autorise les boutons
    if (digitalRead(BP2Pin)==LOW) {                                 // ============================
      Serial.print("BP2 ");
      CptTco=0;
      while (not digitalRead(BP2Pin)==HIGH) {                       // Appui long sur BP2
        if (CptTco<255) CptTco++;
        if (CptTco%10==0) Serial.print(CptTco);Serial.print(" ");
        if (CptTco==CptOnOffLimit) {                                // On/Off des BP
  #ifdef USE_LCD
          lcdDisplay(lcd_Infol,lcd_DccAdc,"S");
  #endif
          BP_actif=!BP_actif;
          EtatBP();
        }
  #ifdef USE_LCD
        else if (CptTco==CptTcoLimit) lcdDisplay(lcd_Infol,lcd_DccAdc,"* ");
  #endif
        delay(50);
      }
      Serial.println();
      
#if defined(UseRedirections)
      ActivateId(CV_Base_REDIR_BP+1);                               // Redirection des BP vers les fonctions
#elif defined(UseBasicRedir)
      if (BP_actif) {
        BP[Dcc.getCV(CV_Base_REDIR_BP)]=(CptTco<CptTcoLimit);       // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#else
      if (BP_actif) {
        BP[9]=(CptTco<CptTcoLimit);                                 // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#endif
    } else if (digitalRead(BP3Pin)==LOW && BP_actif) {              // ============================
      Serial.print("BP3 ");
      CptTco=0;
      while (not digitalRead(BP3Pin)==HIGH) {                       // Appui long sur BP3
        if (CptTco<255) CptTco++;
        if (CptTco%10==0) Serial.print(CptTco);Serial.print(" ");
  #ifdef USE_LCD
        if (CptTco==CptTcoLimit) lcdDisplay(lcd_Infol,lcd_DccAdc,"* ");
  #endif
        delay(50);
      }
      Serial.println();

#if defined(UseRedirections)
      ActivateId(CV_Base_REDIR_BP+2);                               // Redirection des BP vers les fonctions
#elif defined(UseBasicRedir)
      if (BP_actif) {
        BP[Dcc.getCV(CV_Base_REDIR_BP)+1]=(CptTco<CptTcoLimit);     // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#else
      if (BP_actif) {
         BP[10]=(CptTco<CptTcoLimit);                               // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#endif
    } else if (analogRead(OccPin)<=512 && BP_actif) {               // BP4 occupation canton / volume
      CptTco=0;
      Serial.println("c ");
      while (analogRead(OccPin)<512) {
        if (CptTco<255) CptTco+=1;
        if (CptTco%10==0) Serial.print(CptTco);Serial.print(" ");
  #ifdef USE_LCD
        if (CptTco==CptTcoLimit) lcdDisplay(lcd_Infol,lcd_DccAdc,"* ");
  #endif
        delay(50);
      }
#if defined(UseRedirections)
      ActivateId(CV_Base_REDIR_BP+3);                               // Redirection des BP vers les fonctions
#elif defined(UseBasicRedir)
      if (BP_actif) {
        BP[Dcc.getCV(CV_Base_REDIR_BP)+2]=(CptTco<CptTcoLimit);     // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#else
      if (BP_actif) {
        BP[11]=(CptTco<CptTcoLimit);                                // Affectation des dernieres fonctions aux BP
        CptCassePieds=DelaiCassePieds;
      }
#endif
    } else {                                                        // AK : Lecture ADC sur AdKeyPin & decodage du BP
      unsigned int val=analogRead(AdKeyPin);                        // Detection d'un appui a identifier
      if (val > 250) {                                              // necessaire pour stabiliser la lecture
        delay(100);
        val=analogRead(AdKeyPin);                                   // Lecture ADC du bouton
        // Serial.print(F("AnalogRead : ")); Serial.println(val);
        int n=0;
        while (val<SeuilADKEY[n] && n<(sizeof(SeuilADKEY))/2) n++;
        if (n<(sizeof(SeuilADKEY))/2) {
          CptTco=0;
          Serial.print(F("ADKEY = "));
          Serial.print(n+1); Serial.print("   ");
          while (analogRead(AdKeyPin)>250) {                        // Difference entre appui court et long
            if (CptTco<255) CptTco+=1;
            if (CptTco%5==0) Serial.print(CptTco);Serial.print(" ");
  #ifdef USE_LCD
            if (CptTco==CptTcoLimit) lcdDisplay(lcd_Infol,lcd_DccAdc,"* ");
  #endif
            delay(50);
          }
          Serial.println(CptTco);

#if defined(UseRedirections)
          ActivateId(CV_Base_REDIR_AK+n);                           // Redirection des AK vers les fonctions
#elif defined(UseBasicRedir)
          if (BP_actif) {
            BP[Dcc.getCV(CV_Base_REDIR_AK+n)]=(CptTco<CptTcoLimit); // Affectation des premieres fonctions aux AK
            CptCassePieds=DelaiCassePieds;                          // BP[id]=(CptTco>CptTcoLimit) ? 0: 1;

          }
#else
          if (BP_actif) {
            BP[8+n]=(CptTco<CptTcoLimit);                           // Affectation des dernieres fonctions aux BP
            CptCassePieds=DelaiCassePieds;
          }
#endif
        }
      }
    }
#endif
  }

  //  Recherche de commandes USB  =================================================================
#if defined(USE_USB)                                                // Commandes USB
  //  Commandes USB:
  if (stringComplete) {
    Commandes(inputString);
    inputString = "";
    stringComplete = false;
  }
#endif

  //  Routine de service  =========================================================================
  ManageCassePieds();

#ifndef SlowMotionSrv
  for (uint8_t k=0;k<8;k++) {                                     // Mise a jour des servomoteurs Bibliotheque Servo
    if (BPs[k].Type==TypeServo && BPs[k].Objectif!=BPs[k].Angle) BPs[k].Angle=StepServo(k,BPs[k].Angle);
  }
#endif

  currentMillis = millis();
  ManageBlink();                                                  // Clignotement des leds TypeBlinkLed

  // Extinction Temporisée des leds
  if (currentMillis - pMillis >= pMillisLimit) {
    pMillis = currentMillis;
    // Led LED_BUILTIN 13
    if (digitalRead(LED_BUILTIN)==LOW) {
#ifdef SlowMotionSrv
      digitalWrite(LED_BUILTIN,HIGH);
#else
      SoftPWMSet(LED_BUILTIN,255);
#endif
    } else {
#ifdef SlowMotionSrv
      digitalWrite(LED_BUILTIN,LOW);
#else
      SoftPWMSet(LED_BUILTIN,0);
#endif
    }
    // fonctions
    for (uint8_t n=0; n<8; n++) {
      if (Tp[n]>0) {
        Tp[n]--;
        uint8_t Nr=BPs[n].Pin; //Dcc.getCV(CV_Base_8Fct+10*id);     // Pin
        Serial.print(F(" LED J"));
///        Serial.print(Nr);
        Serial.print(JPin(Nr));
        Serial.print(" ");
        if (Tp[n]==0) {
          pinMode(Nr, OUTPUT);
          delay(1);
          Serial.println(F(" Off"));
//          digitalWrite(Nr,LOW);                                   // Digital LED
//          if (BPs[n].Max==BPs[n].Min || Nr==4 || Nr==12) {        // PWM si on à Min <> Max et PWM autorisé (ni J2=4 ni J8=12), On/Off sinon
#ifdef SlowMotionSrv
          if (BPs[n].Max==BPs[n].Min || (Nr!=3 && Nr!=11)) {        // PWM si on à Min <> Max et PWM autorisé (ni J2=4 ni J8=12), On/Off sinon
#else
          if (BPs[n].Max==BPs[n].Min) {                             // PWM si on à Min <> Max et PWM autorisé (ni J2=4 ni J8=12), On/Off sinon
#endif
            digitalWrite(Nr,LOW);                                   // Digital LED
          } else {
#ifdef SlowMotionSrv
            analogWrite(Nr,BPs[n].Min);                           // Analog LED Min   CV 52+10*i ou Led Max    CV 103+10*i
#else
            SoftPWMSet(Nr,BPs[n].Min);                            // Analog LED Min   CV 52+10*i ou Led Max    CV 103+10*i
#endif
          }
  #ifdef USE_LCD
          lcdDisplay(lcd_Infol,lcd_Infoc,String(n+1)+ " Off");
  #endif
          BP[n]=false;
        } else {
          Serial.println(Tp[n]);
        }
  #ifdef USE_LCD
        lcdDisplay(lcd_Infol,7,"T   ");
        lcdDisplay(lcd_Infol,8,String(Tp[n]));
  #endif
      }
    }
  }

  // Execution des actions collectées =============================================================
  DoAction();
  ManageMP3();                                                      // Verification fin MP3

#if defined(SlowMotionSrv)
  // Actualisations de la position des servomoteurs : =============================================
  SlowMotionServo::update();
  delay(1);
#endif
}
